<?php
/**
 * @version     $Id$
 * @package     Jumbotron
 * @subpackage  com_jumbotron
 * @copyright   Copyright 2010 - 2011 Jumbotron Sports. All rights reserved.
 * @license     GNU General Public License version 2 or later.
 */

// No direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');

/**
 * MultiGames model.
 *
 * @package     Jumbotron
 * @subpackage  com_jumbotron
 * @since       2.7.0
 */
class JumbotronModelMultiGames extends JModelList
{
	/**
	 * Constructor override.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @return  JumbotronModelMultiGames
	 * @since   2.7.0
	 * @see     JModelList
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'a.id',
				'game_date', 'a.game_date',
				'home_team', 'a.home_team',
				'away_team', 'a.away_team',
				'visitor_two', 'a.visitor_two',
				'visitor_three', 'a.visitor_three',
				'visitor_four', 'a.visitor_four',
			 	'visitor_five', 'a.visitor_five',
				'visitor_six', 'a.visitor_six',
				'visitor_seven', 'a.visitor_seven',
				'visitor_eight', 'a.visitor_eight',
				'visitor_nine', 'a.visitor_nine',
				'alias', 'a.alias',
				'checked_out', 'a.checked_out',
				'checked_out_time', 'a.checked_out_time',
				'catid', 'a.catid', 'category_title',
				'published', 'a.published',
				'access', 'a.access', 'access_level',
				'ordering', 'a.ordering',
				'language', 'a.language',
				'created_time', 'a.created_time',
				'created_user_id', 'a.created_user_id',
				'modified_time', 'a.modified_time',
				'modified_user_id', 'a.modified_user_id',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 
	 * @param   string  $ordering   An optional ordering field.
	 * @param   string  $direction  An optional direction (asc|desc).
	 *
	 * @return  void
	 * @since   2.7.0
	 */
	protected function populateState($ordering = 'a.game_date', $direction = 'asc')
	{
		// Initialise variables.
		$app = JFactory::getApplication();

		$value = $app->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $value);

		$value = $app->getUserStateFromRequest($this->context.'.filter.access', 'filter_access', 0, 'int');
		$this->setState('filter.access', $value);

		$value = $app->getUserStateFromRequest($this->context.'.filter.published', 'filter_published', '');
		$this->setState('filter.published', $value);

		$value = $app->getUserStateFromRequest($this->context.'.filter.category_id', 'filter_category_id');
		$this->setState('filter.category_id', $value);

		$value = $app->getUserStateFromRequest($this->context.'.filter.language', 'filter_language', '');
		$this->setState('filter.language', $value);

		// Set list state ordering defaults.
		parent::populateState($ordering, $direction);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 * @since   2.7.0
	 */
	protected function getListQuery()
	{
		// Initialise variables.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.id, a.game_date, a.home_team, a.away_team, a.alias, a.checked_out, a.checked_out_time, a.category_id,' .
				'a.published, a.access, a.created_time, a.ordering, a.language'
			)
		);
		$query->from('#__jumbotron_games AS a');
		
		//Join over the home_team
		$query->select('h.team AS home_team');
		$query->join('LEFT', '#__jumbotron_teams AS h ON h.id = a.home_team');
		
		//Join over the away_team
		$query->select('v.team AS away_team');
		$query->join('LEFT', '#__jumbotron_teams AS v ON v.id = a.away_team');

		//Join over the visitor_two
		$query->select('vii.team AS visitor_two');
		$query->join('LEFT', '#__jumbotron_teams AS vii ON vii.id = a.visitor_two');

				//Join over the visitor_three
		$query->select('viii.team AS visitor_three');
		$query->join('LEFT', '#__jumbotron_teams AS viii ON viii.id = a.visitor_three');

				//Join over the visitor_four
		$query->select('viv.team AS visitor_four');
		$query->join('LEFT', '#__jumbotron_teams AS viv ON viv.id = a.visitor_four');

				//Join over the visitor_five
		$query->select('vv.team AS visitor_five');
		$query->join('LEFT', '#__jumbotron_teams AS vv ON vv.id = a.visitor_five');

				//Join over the visitor_six
		$query->select('vvi.team AS visitor_six');
		$query->join('LEFT', '#__jumbotron_teams AS vvi ON vvi.id = a.visitor_six');

				//Join over the visitor_seven
		$query->select('vvii.team AS visitor_seven');
		$query->join('LEFT', '#__jumbotron_teams AS vvii ON vvii.id = a.visitor_seven');

				//Join over the visitor_eight
		$query->select('vviii.team AS visitor_eight');
		$query->join('LEFT', '#__jumbotron_teams AS vviii ON vviii.id = a.visitor_eight');

				//Join over the visitor_nine
		$query->select('vix.team AS visitor_nine');
		$query->join('LEFT', '#__jumbotron_teams AS vix ON vix.id = a.visitor_nine');

		// Join over the language
		$query->select('l.title AS language_title');
		$query->join('LEFT', '`#__languages` AS l ON l.lang_code = a.language');

		// Join over the users for the checked out user.
		$query->select('uc.name AS editor');
		$query->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over the asset groups.
		$query->select('ag.title AS access_level');
		$query->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');

		// Join over the categories.
		$query->select('c.title AS category_title');
		$query->join('LEFT', '#__categories AS c ON c.id = a.category_id');

		// Join over the users for the author.
		$query->select('ua.name AS author_name');
		$query->join('LEFT', '#__users AS ua ON ua.id = a.created_user_id');

		// Where the game_date has gone by
		$query->where('a.game_date > NOW()');
		
		// Where there are only 2 teams competing
		$query->where('a.visitor_two > 0');

		// Filter by search in game
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$query->where('a.id = '.(int) substr($search, 3));
			} else {
				$search = $db->Quote('%'.$db->getEscaped($search, true).'%');
				$query->where('(a.game_date LIKE '.$search.
								' OR a.alias LIKE '.$search.
								' OR h.team LIKE '.$search.
								' OR v.team LIKE '.$search.
								' OR vii.team LIKE '.$search.
								' OR viii.team LIKE '.$search.
								' OR viv.team LIKE '.$search.
								' OR vv.team LIKE '.$search.
								' OR vvi.team LIKE '.$search.
								' OR vvii.team LIKE '.$search.
								' OR vviii.team LIKE '.$search.
								' OR vix.team LIKE '.$search.')');
							}
		}

		// Filter by access level.
		if ($access = $this->getState('filter.access')) {
			$query->where('a.access = ' . (int) $access);
		}

		// Filter by published state
		$published = $this->getState('filter.published');
		if (is_numeric($published)) {
			$query->where('a.published = ' . (int) $published);
		} else if ($published === '') {
			$query->where('(a.published = 0 OR a.published = 1)');
		}

		// Filter by a single or group of categories.
		$categoryId = $this->getState('filter.category_id');
		if (is_numeric($categoryId)) {
			$query->where('a.category_id = '.(int) $categoryId);
		}
		else if (is_array($categoryId)) {
			JArrayHelper::toInteger($categoryId);
			$categoryId = implode(',', $categoryId);
			$query->where('a.category_id IN ('.$categoryId.')');
		}

		// Filter on the language.
		if ($language = $this->getState('filter.language')) {
			$query->where('a.language = '.$db->quote($language));
		}

		// Add the list ordering clause.
		$orderCol	= $this->state->get('list.ordering');
		$orderDirn	= $this->state->get('list.direction');
		if ($orderCol == 'a.ordering' || $orderCol == 'category_title') {
			$orderCol = 'category_title '.$orderDirn.', a.ordering';
		}
		$query->order($db->getEscaped($orderCol.' '.$orderDirn));

		//echo nl2br(str_replace('#__','jos_',$query));
		return $query;
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store id.
	 *
	 * @return  string  A store id.
	 * @since   2.7.0
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id	.= ':'.$this->getState('filter.search');
		$id .= ':'.$this->getState('filter.access');
		$id	.= ':'.$this->getState('filter.published');
		$id	.= ':'.$this->getState('filter.category_id');
		$id	.= ':'.$this->getState('filter.language');

		return parent::getStoreId($id);
	}

}